package com.mt.wms.empty.controller;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.google.gson.JsonObject;
import com.mt.wms.core.base.BaseController;
import com.mt.wms.core.constants.CommonConstant;
import com.mt.wms.core.dal.entity.ParEleValue;
import com.mt.wms.core.dal.service.ParEleValueServiceBiz;
import com.mt.wms.core.vo.R;
import com.mt.wms.empty.params.ElectricQueryParam;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.PostConstruct;
import javax.validation.groups.Default;
import java.time.DayOfWeek;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.temporal.ChronoUnit;
import java.time.temporal.TemporalAdjusters;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

/**
 * @Author: liguanghao
 * @Date: 2021/12/16 11:20
 * @Version 1.0
 */
@RestController
@RequestMapping(CommonConstant.API_MODULE_BASE + "electric")
@Slf4j
@Api(value = "电能消耗相关接口", tags = "电能消耗相关接口", hidden = false)
public class ElectricController extends BaseController {


    //@PostConstruct
    public void test(){
        LocalDateTime time=LocalDateTime.of(2021, 3, 31, 0, 0, 0);
        Float f=29000F;

        LocalDate timeDate=time.toLocalDate();
        LocalDateTime timeEnd = LocalDateTime.of(timeDate, LocalTime.MAX).minus(5,ChronoUnit.SECONDS);
        for (int i = 0; i < 10; i++) {

            ParEleValue parEleValue=new ParEleValue();
            parEleValue.setDateType(2);
            parEleValue.setParId(1);
            parEleValue.setKilnId(1);
            parEleValue.setTotalBat(Float.valueOf(String.valueOf(f+(Math.random()*24000))));
            parEleValue.setCreateTime(timeEnd);
            timeEnd=timeEnd.plus(1,ChronoUnit.MONTHS);
            f=parEleValue.getTotalBat();
            parEleValueServiceBiz.save(parEleValue);
            System.out.println(parEleValue);
        }

    }

    public void test1(){

    }

    @Autowired
    private ParEleValueServiceBiz parEleValueServiceBiz;

    @PostMapping(value = "eletric")
    @ApiOperation(value = "根据窑炉id获取电能消耗情况")
    private R<String> list(@Validated({Default.class}) @RequestBody ElectricQueryParam electricQueryParam) {
        Long kiln = electricQueryParam.getKilnId();
        //窑炉id为0代表全部，存储的时候计算全部
            JSONObject jsonObject=new JSONObject();
            //指定时间节点
            LocalDate timeDate = electricQueryParam.getTime().toLocalDate();
            LocalDateTime timeStart = LocalDateTime.of(timeDate, LocalTime.MIN);
            LocalDateTime timeEnd = LocalDateTime.of(timeDate, LocalTime.MAX);
            //当前时间节点
            LocalDateTime todayStart = LocalDateTime.of(LocalDate.now(), LocalTime.MIN);
            LocalDateTime todayEnd = LocalDateTime.of(LocalDate.now(), LocalTime.MAX);

            //总电能消耗
            List<ParEleValue> todayEleValueList = parEleValueServiceBiz.list(new QueryWrapper<ParEleValue>()
                    .eq("kiln_id", kiln)
                    .between("create_time", todayStart, todayEnd)
                    .orderByDesc("create_time"));
            Float countEleValue = todayEleValueList.get(0).getTotalBat();
            jsonObject.put("countEle",countEleValue);

            //昨日电能消耗
            LocalDateTime yesterdayStart = LocalDateTime.of(LocalDate.now().plus(1, ChronoUnit.DAYS), LocalTime.MIN);
            LocalDateTime yesterdayEnd = LocalDateTime.of(LocalDate.now().plus(1, ChronoUnit.DAYS), LocalTime.MAX);
            List<ParEleValue> yesterdayEleValueList = parEleValueServiceBiz.list(new QueryWrapper<ParEleValue>()
                    .eq("kiln_id", kiln)
                    .between("create_time", yesterdayStart, yesterdayEnd)
                    .eq("date_type",0)
                    .orderByAsc("create_time"));
            Float yesterdayTotalBat= yesterdayEleValueList.get(yesterdayEleValueList.size()-1).getTotalBat()-yesterdayEleValueList.get(0).getTotalBat();
            jsonObject.put("yesterdayCountEle",yesterdayTotalBat);

            //指定日期当天的电能消耗
            if (electricQueryParam.getDateType()==0){
                List<ParEleValue> dateEleValueList = parEleValueServiceBiz.list(new QueryWrapper<ParEleValue>()
                        .eq("kiln_id", kiln)
                        .between("create_time", timeStart, timeEnd)
                        .orderByAsc("create_time"));
                //总耗电量
                Float dateCountEle=0F;
                if (dateEleValueList.size()!=0){
                    dateCountEle= dateEleValueList.get(dateEleValueList.size()-1).getTotalBat()-dateEleValueList.get(0).getTotalBat();
                }
                jsonObject.put("dateCountEle",dateCountEle);
                //分时消耗量
                Map<String,Float> timeJsonObject=new LinkedHashMap();
                for (int i = 0; i < dateEleValueList.size() - 1; ++i) {
                    // TODO: 2021/12/16
                    float eleValue = dateEleValueList.get(i).getTotalBat() - dateEleValueList.get(i - 1).getTotalBat();
                    int j=i+1;
                    if (j%2==0){
                        String time=(j-1)/2+":30";
                        timeJsonObject.put(time,eleValue);
                    }else {
                        String time=(j-1)/2+":00";
                        timeJsonObject.put(time,eleValue);
                    }
                }
                jsonObject.put("timeEle",timeJsonObject);
            }
            //指定日期当周的每天电能消耗
            if(electricQueryParam.getDateType()==1){
                LocalDate monday = timeDate.with(DayOfWeek.MONDAY);
                LocalDate sunday = timeDate.with(DayOfWeek.SUNDAY);
                List<ParEleValue> timeWeekEleValueList = parEleValueServiceBiz.list(new QueryWrapper<ParEleValue>()
                        .eq("kiln_id", kiln)
                        .between("create_time", monday, sunday.plusDays(1))
                        .eq("date_type", 1)
                        .orderByAsc("create_time"));
                //总耗电量
                Float dateCountEle=0F;
                if (timeWeekEleValueList.size()!=0){
                    dateCountEle= timeWeekEleValueList.get(timeWeekEleValueList.size()-1).getTotalBat()-timeWeekEleValueList.get(0).getTotalBat();
                }
                jsonObject.put("dateCountEle",dateCountEle);
                Map<String,Float> timeJsonObject=new LinkedHashMap();
                for (ParEleValue parEleValue:timeWeekEleValueList
                     ) {
                    LocalDate localDate = parEleValue.getCreateTime().toLocalDate();
                    timeJsonObject.put(localDate.toString(),parEleValue.getTotalBat());
                }
                jsonObject.put("timeEle",timeJsonObject);
            }
            //指定日期当月的每天电能消耗
            if(electricQueryParam.getDateType()==2){
                LocalDate firstday = LocalDate.of(timeDate.getYear(), timeDate.getMonthValue(), 1);
                LocalDate lastDay = timeDate.with(TemporalAdjusters.lastDayOfMonth());
                List<ParEleValue> timeMonthEleValueList = parEleValueServiceBiz.list(new QueryWrapper<ParEleValue>()
                        .eq("kiln_id", kiln)
                        .between("create_time", firstday, lastDay.plusDays(1))
                        .eq("date_type", 1)
                        .orderByAsc("create_time"));
                //总耗电量
                Float dateCountEle=0F;
                if (timeMonthEleValueList.size()!=0){
                    dateCountEle= timeMonthEleValueList.get(timeMonthEleValueList.size()-1).getTotalBat()-timeMonthEleValueList.get(0).getTotalBat();
                }
                jsonObject.put("dateCountEle",dateCountEle);
                Map<String,Float> timeJsonObject=new LinkedHashMap();
                for (ParEleValue parEleValue:timeMonthEleValueList
                ) {
                    LocalDate localDate = parEleValue.getCreateTime().toLocalDate();
                    timeJsonObject.put(localDate.toString(),parEleValue.getTotalBat());
                }
                jsonObject.put("timeEle",timeJsonObject);
            }
            //指定日期当年的每月电能消耗
            if(electricQueryParam.getDateType()==2){
                LocalDate firstDay = timeDate.with(TemporalAdjusters.firstDayOfYear());
                LocalDate lastDay = timeDate.with(TemporalAdjusters.lastDayOfYear());
                List<ParEleValue> timeYearEleValueList = parEleValueServiceBiz.list(new QueryWrapper<ParEleValue>()
                        .eq("kiln_id", kiln)
                        .between("create_time", firstDay, lastDay.plusDays(1))
                        .eq("date_type", 2)
                        .orderByAsc("create_time"));
                //总耗电量
                Float dateCountEle=0F;
                if (timeYearEleValueList.size()!=0){
                    dateCountEle= timeYearEleValueList.get(timeYearEleValueList.size()-1).getTotalBat()-timeYearEleValueList.get(0).getTotalBat();
                }
                jsonObject.put("dateCountEle",dateCountEle);
                Map<String,Float> timeJsonObject=new LinkedHashMap();
                for (ParEleValue parEleValue:timeYearEleValueList
                ) {
                    LocalDate localDate = parEleValue.getCreateTime().toLocalDate();
                    timeJsonObject.put(localDate.toString(),parEleValue.getTotalBat());
                }
                jsonObject.put("timeEle",timeJsonObject);
            }
        return successful(jsonObject.toJSONString());
    }
}
