package com.mt.wms.empty.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.mt.wms.core.base.BaseService;
import com.mt.wms.core.dal.entity.InStockInfo;
import com.mt.wms.core.dal.entity.InStockInfoHis;
import com.mt.wms.core.dal.entity.RunTask;
import com.mt.wms.core.dal.entity.VehicleInfo;
import com.mt.wms.core.dal.service.InStockInfoHisServiceBiz;
import com.mt.wms.core.dal.service.InStockInfoServiceBiz;
import com.mt.wms.core.dal.service.RunTaskServiceBiz;
import com.mt.wms.core.dal.service.VehicleInfoServiceBiz;
import com.mt.wms.core.dal.service.impl.InStockInfoHisServiceBizImpl;
import com.mt.wms.core.vo.R;
import com.mt.wms.empty.params.OutStockParam;
import com.mt.wms.empty.service.OutStockService;
import com.mt.wms.empty.task.AsynRunTaskService;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import javax.validation.constraints.NotNull;
import java.time.LocalDateTime;
import java.util.List;

/**
 * @Author: liguanghao
 * @Date: 2021/12/17 8:52
 * @Version 1.0
 */
@Service
@Transactional
public class OutStockServiceImpl extends BaseService implements OutStockService {

    @Resource
    private VehicleInfoServiceBiz vehicleInfoServiceBiz;
    @Resource
    private AsynRunTaskService asynRunTaskService;
    @Resource
    private InStockInfoServiceBiz inStockInfoServiceBiz;
    @Resource
    private RunTaskServiceBiz runTaskServiceBiz;


    @Override
    public R<String> outTask(OutStockParam outStockParam) {
        // TODO: 2021/12/17 出库
        Long locationId = outStockParam.getLocaltionId();
        //提升台
        Integer code = outStockParam.getCode();
        //查询是否有空闲车辆，若有。占用车辆，若无，返回暂无可用车辆
        List<VehicleInfo> vehicleInfoList = vehicleInfoServiceBiz.list(new QueryWrapper<VehicleInfo>()
                .eq(VehicleInfo.STATUS, 0)
                .eq(VehicleInfo.VALID, 1));
        if (vehicleInfoList.size()>0){
            VehicleInfo vehicleInfo = vehicleInfoList.get(0);
            vehicleInfo.setStatus(1);
            vehicleInfo.setUpdateTime(LocalDateTime.now());
            vehicleInfoServiceBiz.updateById(vehicleInfo);
        }else {
            return R.failed("暂无可用车辆!请稍后重试！");
        }
        //生成调度任务
        RunTask runTask=new RunTask();
        runTask.setBeginTime(LocalDateTime.now());
        runTask.setVehicleId(vehicleInfoList.get(0).getId());
        // TODO: 2021/12/22 库位id对应的位置 ，提升平台对应的位置
        runTask.setStartPosition("库位位置");
        runTask.setEndPosition("提升台位置");
        setCommonField(runTask);
        runTaskServiceBiz.save(runTask);
        //异步调用车辆
        asynRunTaskService.asynRunOutStock(runTask.getId(),locationId);
        //修改库位状态
        InStockInfo inStockInfo = inStockInfoServiceBiz
                .getOne(new QueryWrapper<InStockInfo>().eq("Location_id", locationId));
        inStockInfo.setStatus(1);
        inStockInfoServiceBiz.updateById(inStockInfo);
        return R.ok("操作成功");
    }
}
