package com.mt.wms.empty.task;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.mt.wms.core.base.BaseService;
import com.mt.wms.core.dal.entity.*;
import com.mt.wms.core.dal.service.*;
import com.mt.wms.core.utils.IDGenerator;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.time.LocalDateTime;

/**
 * @Author: liguanghao
 * @Date: 2021/11/18 20:27
 * @Version 1.0
 */
@Service
@Transactional
public class AsynRunTaskService extends BaseService {
    private final static Logger logger = LoggerFactory.getLogger(AsynRunTaskService.class);

    @Resource
    private CurrTaskServiceBiz currTaskServiceBiz;
    @Resource
    private RunTaskServiceBiz runTaskServiceBiz;
    @Resource
    private LocationServiceBiz locationServiceBiz;
    @Resource
    private InStockInfoServiceBiz inStockInfoServiceBiz;
    @Resource
    private InStockInfoHisServiceBiz inStockInfoHisServiceBiz;
    //窑炉可用未满的情况下，调用车辆起点为提升台终点为窑炉
    @Async("asyncServiceExecutor")
    public void asynRunTask(Long currTaskId,Long vehicleId){
        CurrTask currTask = currTaskServiceBiz.getById(currTaskId);
        //新建一条执行任务的关系表存放任务执行信息
        RunTask runTask=new RunTask();
        runTask.setTaskId(currTask.getId());
        runTask.setTaskCode(currTask.getTaskCode());
        runTask.setVehicleId(vehicleId);
        runTask.setStartPosition(currTask.getStartPosition());
        runTask.setEndPosition(currTask.getTargetPosition());
        setCommonField(runTask);
        runTask.setBeginTime(LocalDateTime.now());
        runTaskServiceBiz.save(runTask);
        // TODO: 2021/12/14  调用车辆填入起终点,得到车辆执行结果

        //执行成功,托盘进炉
        if (true){
            //更新调度表状态为完成
            runTask.setUpdateTime(LocalDateTime.now());
            runTask.setEndTime(LocalDateTime.now());
            runTask.setStatus(2);
            runTaskServiceBiz.updateById(runTask);
            //更新currTask表状态
            currTask.setIsIn(1);
            currTask.setRunTaskId(runTask.getId());
            currTask.setUpdateTime(LocalDateTime.now());
            currTaskServiceBiz.updateById(currTask);
        }else {
            runTask.setStatus(3);
            runTask.setUpdateTime(LocalDateTime.now());
            runTaskServiceBiz.updateById(runTask);
            logger.info("任务 "+currTask.getTaskCode()+" 车辆从提升平台到窑炉过程中失败。");
            // TODO: 2021/12/14 websocket推送到前端
        }
    }
    //窑炉已满，调用车辆起点为提升台，终点为缓存区
    @Async("asyncServiceExecutor")
    public void asynRunTaskToWarehouse(Long currTaskId,Long vehicleId){
        CurrTask currTask = currTaskServiceBiz.getById(currTaskId);
        Location location = locationServiceBiz.list(new QueryWrapper<Location>()
                .eq(Location.STATUS, 1)
                .eq(Location.VALID, 1)).get(0);
        //新建一条执行任务的关系表存放任务执行信息,终点为缓存区空闲库位
        RunTask runTask=new RunTask();
        setCommonField(runTask);
        runTask.setTaskId(currTask.getId());
        runTask.setTaskCode(currTask.getTaskCode());
        runTask.setVehicleId(vehicleId);
        runTask.setBeginTime(LocalDateTime.now());
        runTask.setStartPosition(currTask.getStartPosition());
        // TODO: 2021/12/28 location对应的点位
        runTask.setEndPosition(location.getCode());
        runTaskServiceBiz.save(runTask);
        //更新 location 表对应库位状态为占用
        location.setStatus(1);
        location.setUpdateTime(LocalDateTime.now());
        locationServiceBiz.updateById(location);
        //添加一条库位详情到in_stock_info表
        InStockInfo inStockInfo=new InStockInfo();
        inStockInfo.setStatus(1);
        inStockInfo.setInterCode(IDGenerator.gen("HCT", "yyyyMMddHHmm", 2, "WAREHOUSE_CODE"));
        inStockInfo.setTaskId(currTask.getId());
        inStockInfo.setTaskCode(currTask.getTaskCode());
        inStockInfo.setKilnId(currTask.getKilnId());
        inStockInfo.setKilnName(currTask.getKilnName());
        inStockInfo.setLocationId(location.getId());
        inStockInfo.setLocationName(location.getLocationNameAlias());
        inStockInfo.setType(1);
        setCommonField(inStockInfo);
        inStockInfoServiceBiz.save(inStockInfo);
        //修改currTask的是否缓存,库位id,库位名称
        currTask.setIsCache(1);
        currTask.setCacheRunTaskId(runTask.getId());
        currTask.setLocationId(location.getId());
        currTask.setLocationName(location.getLocationNameAlias());
        currTaskServiceBiz.updateById(currTask);
        // TODO: 2021/12/14  调用车辆填入起终点,得到车辆执行结果,填入任务号（待确认）

        if (true){
            //更新关系表状态为完成
            runTask.setUpdateTime(LocalDateTime.now());
            runTask.setStatus(2);
            runTask.setEndTime(LocalDateTime.now());
            runTaskServiceBiz.updateById(runTask);
            //更新currTask表
            currTask.setCacheSatatus(2);
            currTask.setUpdateTime(LocalDateTime.now());
            currTaskServiceBiz.updateById(currTask);
            //更新in_stock_info表
            inStockInfo.setStatus(2);
            inStockInfo.setInTime(LocalDateTime.now());
            inStockInfoServiceBiz.updateById(inStockInfo);
        }else {
            runTask.setStatus(3);
            runTask.setUpdateTime(LocalDateTime.now());
            runTaskServiceBiz.updateById(runTask);
            logger.info("任务 "+currTask.getTaskCode()+" 车辆从提升平台到缓存区过程中失败。");
            // TODO: 2021/12/14 websocket推送到前端
        }
    }
    //窑炉加工完成，调用车辆入库到缓存区
    @Async("asyncServiceExecutor")
    public void asynRunTaskForKilnToWarehouse(Long currTaskId,Long vehicleId){
        CurrTask currTask = currTaskServiceBiz.getById(currTaskId);
        // TODO: 2021/12/28 location排序
        Location location = locationServiceBiz.list(new QueryWrapper<Location>()
                .eq(Location.STATUS, 1)
                .eq(Location.VALID, 1)).get(0);
        currTask.setOutStartPosition(currTask.getTargetPosition());
        // TODO: 2021/12/28 location对应的点位
        currTask.setOutTargetPosition(location.getCode());
        currTaskServiceBiz.updateById(currTask);
        //新建一条执行任务的关系表存放任务执行信息,终点为缓存区空闲库位
        RunTask runTask=new RunTask();
        setCommonField(runTask);
        runTask.setTaskId(currTask.getId());
        runTask.setTaskCode(currTask.getTaskCode());
        runTask.setVehicleId(vehicleId);
        runTask.setBeginTime(LocalDateTime.now());
        runTask.setStartPosition(currTask.getOutStartPosition());
        runTask.setEndPosition(currTask.getOutTargetPosition());
        runTaskServiceBiz.save(runTask);
        //更新 location 表对应库位状态为占用
        location.setStatus(1);
        location.setUpdateTime(LocalDateTime.now());
        locationServiceBiz.updateById(location);
        currTask.setOutRunTaskId(runTask.getId());
        currTaskServiceBiz.updateById(currTask);
        //添加一条库位详情到in_stock_info表
        InStockInfo inStockInfo=new InStockInfo();
        inStockInfo.setStatus(1);
        inStockInfo.setInterCode(IDGenerator.gen("HCT", "yyyyMMddHHmm", 2, "WAREHOUSE_CODE"));
        inStockInfo.setTaskId(currTask.getId());
        inStockInfo.setTaskCode(currTask.getTaskCode());
        inStockInfo.setKilnId(currTask.getKilnId());
        inStockInfo.setKilnName(currTask.getKilnName());
        inStockInfo.setLocationId(location.getId());
        inStockInfo.setLocationName(location.getLocationNameAlias());
        inStockInfo.setType(0);
        setCommonField(inStockInfo);
        inStockInfoServiceBiz.save(inStockInfo);
        // TODO: 2021/12/14  调用车辆填入起终点,得到车辆执行结果,填入任务号（待确认）
        if (true){
            //更新关系表状态为完成
            runTask.setUpdateTime(LocalDateTime.now());
            runTask.setStatus(2);
            runTask.setEndTime(LocalDateTime.now());
            runTaskServiceBiz.updateById(runTask);
            //更新currTask表
            currTask.setIsOut(2);
            currTask.setUpdateTime(LocalDateTime.now());
            currTask.setStatus(2);
            currTaskServiceBiz.updateById(currTask);
            //更新in_stock_info表
            inStockInfo.setStatus(2);
            inStockInfo.setInTime(LocalDateTime.now());
            inStockInfoServiceBiz.updateById(inStockInfo);
        }else {
            runTask.setStatus(3);
            runTask.setUpdateTime(LocalDateTime.now());
            runTaskServiceBiz.updateById(runTask);
            logger.info("任务 "+currTask.getTaskCode()+" 车辆从窑炉到缓存区过程中失败。");
            // TODO: 2021/12/14 websocket推送到前端
        }
    }
    //出库。起点为缓存区库位，终点为提升台
    @Async("asyncServiceExecutor")
    public void asynRunOutStock(Long runTaskId,Long locationId){
        //修改调度任务状态
        RunTask runTask = runTaskServiceBiz.getById(runTaskId);
        //修改缓存区状态
        InStockInfo inStockInfo = inStockInfoServiceBiz.getOne(new QueryWrapper<InStockInfo>()
                .eq("location_id", locationId));
        inStockInfo.setStatus(1);
        inStockInfoServiceBiz.updateById(inStockInfo);

        // TODO: 2021/12/14  调用车辆填入起终点,得到车辆执行结果,填入任务号（待确认）

        if (true){
            //更新关系表状态为完成
            runTask.setUpdateTime(LocalDateTime.now());
            runTask.setStatus(2);
            runTask.setEndTime(LocalDateTime.now());
            runTaskServiceBiz.updateById(runTask);
            //更新in_stock_info表
            inStockInfo.setStatus(2);
            inStockInfo.setInTime(LocalDateTime.now());
            inStockInfoServiceBiz.updateById(inStockInfo);
            //缓存区存储历史
            InStockInfoHis inStockInfoHis=new InStockInfoHis();
            BeanUtils.copyProperties(inStockInfo,inStockInfoHis);
            inStockInfoHisServiceBiz.save(inStockInfoHis);
            inStockInfoServiceBiz.removeById(inStockInfo.getId());
            //更新库位状态
            Location location = locationServiceBiz.getById(locationId);
            location.setStatus(0);
            locationServiceBiz.updateById(location);

        }else {
            runTask.setStatus(3);
            runTask.setUpdateTime(LocalDateTime.now());
            runTaskServiceBiz.updateById(runTask);
            logger.info("任务 "+runTask.getTaskCode()+" 车辆从缓存区到提升平台过程中失败。");
            // TODO: 2021/12/14 websocket推送到前端
        }
    }
}
